import { createWriteStream } from 'fs';
import * as path from 'path';
import { FileSystemUtility } from './file-system-utility';
import { OutputMode } from './options';
import { StreamWriter } from './stream-writer';
/**
 * Opens or creates a text file and executes the specified generator callback on this file.
 */
var FileSystemCodeGenerator = /** @class */ (function () {
    function FileSystemCodeGenerator() {
    }
    FileSystemCodeGenerator.generate = function (options, generator) {
        var filePath = options.outputFile;
        // Ensure that the directory exists
        if (options.ensureDirectory) {
            FileSystemUtility.ensureDirectory(path.dirname(filePath));
        }
        // Generate the file
        var flags = options.outputMode === OutputMode.Append ? 'a' : 'w';
        var writeStream;
        try {
            writeStream = createWriteStream(filePath, { flags: flags });
        }
        catch (e) {
            return Promise.reject(e);
        }
        return new Promise(function (resolve, reject) {
            writeStream
                .once('ready', function (fd) {
                var output = new StreamWriter(writeStream, options.regionMarkerFormatter);
                generator(output)
                    .then(function () {
                    writeStream.end();
                    resolve();
                })
                    .catch(function (err) {
                    reject(err);
                });
            });
        });
    };
    return FileSystemCodeGenerator;
}());
export { FileSystemCodeGenerator };
