import { createWriteStream } from 'fs';
import * as path from 'path';
import { FileSystemUtility } from './file-system-utility';
import { OutputMode } from './options';
import { StreamWriter } from './stream-writer';
/**
 * Opens or creates a text file and executes the specified generator callback on this file.
 */
export class FileSystemCodeGenerator {
    static generate(options, generator) {
        const filePath = options.outputFile;
        // Ensure that the directory exists
        if (options.ensureDirectory) {
            FileSystemUtility.ensureDirectory(path.dirname(filePath));
        }
        // Generate the file
        const flags = options.outputMode === OutputMode.Append ? 'a' : 'w';
        let writeStream;
        try {
            writeStream = createWriteStream(filePath, { flags: flags });
        }
        catch (e) {
            return Promise.reject(e);
        }
        return new Promise((resolve, reject) => {
            writeStream
                .once('ready', (fd) => {
                var output = new StreamWriter(writeStream, options.regionMarkerFormatter);
                generator(output)
                    .then(() => {
                    writeStream.end();
                    resolve();
                })
                    .catch(err => {
                    reject(err);
                });
            });
        });
    }
}
